# Out-stream Player

*Definition: Out-stream*  
Publishers without their own video content on a page can use Out-Stream advertisement.
This is a stand-alone video ad unit, typically shown within text contents such as articles and recipes in a large format player. It will autoplay muted on becoming visibile to the user.

## Getting Started

Latest edition of Out-stream player can be found at:  
[https://sdk.smartclip.net/smartoutstream/](https://sdk.smartclip.net/smartoutstream/)

### Minimal Setup

The following example will play automatically when scrolled into visible area.

```javascript
let _outstreamPlayer = new OutstreamPlayer('#targetelement',{
  adRequest:'//adserver.com/creatives/vast.xml?[CACHEBUSTING]'
});
```

Where `#targetelement` can be any CSS selector, if multiple elements are found only the first is used. It'll automatically play when it's scrolled into view.

### Configuration

#### autoplay

`autoplay: false`  
Sets whether the player is automatically started when it reached the [visibilityThreshold](#visibilityThreshold).  
The default is `true`.

#### debug

`debug: true`  
Logging to the browser console is enabled by setting `debug: true`, this will give you runtime information of the underlying ad framework and the player itself.  
Default is `false` to keep your console clean.

#### desiredBitrate

`desiredBitrate: 800`  
You can specify a target bitrate for the creative, higher values will increase video quality but will cost bandwidth. Value is given in kpbs.  
Default is `700`.

#### endingScreen

`endingScreen: false`  
By default the player will not close, but show the ending screen when an advertisement is complete (last frame of the ad and a replay button, if an advertisment comes with an endcard that will be shown). If set to `false` the player will collapse. Some VPAID creatives can cause issues with ending screen or replay behaviour.
Default is `true`.

#### maxAdWidth

`maxAdWidth: 960`  
The player will fill the whole width of the element it gets, to have it narrower a different maximum width can be defined in pixels.  
Default is `800`.

#### minAdWidth

`minAdWidth: 180`  
If the visible area is narrower than this size, no ad will be requested. The value is given in pixels.
Default is `280`.

#### prefetching

`prefetching: true`  
When turned off the player will only start to request the ad when it's scrolled into the visible area, this will cause visible lagging as the response has to finish loading before the player will open.  
The default is `true`.

#### removePlayerOnCapped (deprecated)

`removePlayerOnCapped: true`  
By default all generated OutStreamPlayer HTML is deleted from the DOM; when there's no endingScreen, endCard or creative defined. 
Default id `true`. Only consider using this flag, if the `startOpen` default has been changed as well.

#### skipOffset

`skipOffset: 0`  
In order to enable skipping from the start set the delay to `0`, to show the skip button after 5 seconds set it to `5`. Setting a general skipOffset is discouraged.
Note that linear creatives carrying a skipsoffet attribute will override the general player setting.  
By default the player does not set a general skipoffset, so a skip button will only be shown, if an ad has a specific skipoffset attached.

#### startOpen

`startOpen: false`  
Per default the player will start fully expanded, if a valid ad can be played. Setting this option to `false` will trigger an expand animation instead once the player comes into view.
Default is `true`.

#### title

`title: 'Advert [remainingTime]'`  
The player can show a freely definable text, a macro _[remainingTime]_ in this string will be replaced with the remaining play time of the ad in seconds.

#### visibilityThreshold

` visibilityThreshold: 30 `  
Defines the percentage of the player which has to be in the visible area to play and pause the advertisment.
The default is `50`.

#### volume

`volume: 0.5`  
Defines volume that is set after unmuting the player, as default it has full loudness with `1.0`.

### Configuration overrides
A few selected options can be set through the *extension* node in the ad tag.
```xml
<Extensions>
    <Extension name="SmartPlayConfig">
        <autoplay enabled="false"/>
        <visibilityThreshold value="25"/>
    </Extension>
</Extensions>
```

### Callbacks

Do not have to be set, but work similar to callbacks in the previous Out-stream player. The _Start_ and _End_ callbacks are always fired, _Empty_ (onCapped) is added before _End_ if the VAST response does not contain an advertisement.
Parameter `api` is usually not passed outside, mostly remains empty, but will provide VAST Extension details. `element` is the DOM element where the player initialized. `onClickThruCallback` does not carry any parameters. `onEventCallback` fires at certain key points during the player's and ad's lifetime (most regular VAST tracking), starting with _AdTagParsed_ ending with _AdSlotComplete_.

```javascript
onCappedCallback: function onCappedCallback(api, element) {
  console.log('### onCappedCallback ###');
},
onStartCallback: function onStartCallback(api, element) {
  console.log('### onStartCallback ###', api, element);
},
onEndCallback: function onEndCallback(api, element) {
  console.log('### onEndCallback ###');
},
onEventCallback: function onEventCallback(api, element, event) {
  console.log('### ### onEventCallback', api, element, event);
},
onClickThruCallback: function onClickThruCallback() {
  console.log('### onClickThruCallback ###');
}
```

## Improving on the minimal example

Using the full configuration options and callbacks, an extended configuration could be as the following example:

```javascript
let _outstreamPlayer = new OutstreamPlayer('#targetelement', {

  adRequest:'https://ad.sxp.smartclip.net/select?type=vast4&s=testadvertiser.dynamic.adpods&sz=pre&test=newlogo&optout=[optout]&consent=[consent]&ang_ua=[UserAgent]&ang_ip=[IP]&ang_ref=[referrer]&rnd=[CACHEBUSTING]',

  debug: true,

  title: 'The ad ends in [remainingTime] seconds',
  skipOffset: 3,     /* will be overridden by creative skipoffset */

  volume: 0.9,
  desiredBitrate: 1234,
  //-- end startup config

  //*/
  onCappedCallback: function onCappedCallback(api, element) {
    console.log('%c Callback: EMPTY', 'background: #34495e; color: #ecf0f1; font-weight:bold;', api, element);
  },
  onStartCallback: function onStartCallback(api, element) {
    console.log('%c Callback: START', 'background: #34495e; color: #ecf0f1; font-weight:bold;', api, element);
  },
  onEndCallback: function onEndCallback(api, element) {
    console.log('%c Callback: END', 'background: #34495e; color: #ecf0f1; font-weight:bold;', api, element);
  },
  onEventCallback: function onEventCallback(api, element, event) {
    console.log('%c Callback: EVENT >> ' + event.type, 'background: #34495e; color: #ecf0f1; font-weight:bold;', api, element, event);

    // get extension if they're found in the ad tag
    if(event.type === "ExtensionsDetected"){
      console.log('%c Callback: EVENT >> ' + event.type, 'background: #34495e; color: #ecf0f1; font-weight:bold; Extensions:', api.extensions);
    }
  },
  onClickThruCallback: function onClickThruCallback() {
    console.log('%c Callback: CLICK', 'background: #34495e; color: #ecf0f1; font-weight:bold;');
  }
});
```

## Known Issues

- *Click To Play* is not yet configurable
- Setting visibilityThreshold to 100 will cause runtime error

